#if !defined HAVE_TRANSPOSE_BA_H__
#define      HAVE_TRANSPOSE_BA_H__

#include "ds/bitarray.h"
#include "aux2/transpose.h"  // transpose_square()

#include "fxttypes.h"
//#include "bits/bitsperlong.h"


//#define  SRC(k)  (((k)*nc)%n1)  // note: overflow if k*nc >= 2^BITS_PER_LONG
#define  SRC(k)  (((unsigned long long)(k)*nc)%n1)  // note: overflow if k*nc >= 2^64
//#define  DST(k)  (((k)*nr)%n1)
template <typename Type>
void transpose_ba(Type *f, ulong nr, ulong nc, bitarray *ba=0)
// in-place transposition of an  nr X nc  array
// that lies in contiguous memory
//
// use bitarray for tagging moved elements
//
{
    if ( 1>=nr )  return;
    if ( 1>=nc )  return;

    if ( nr==nc )  transpose_square(f, nr);
    else
    {
        const ulong n1 = nr * nc - 1;

        bitarray *tba = 0;
        if ( 0==ba )  tba = new bitarray(n1);
        else          tba = ba;

        for (ulong k=0;  k<n1;  k=tba->next_clear(++k) )  //  0 and n1 are fixed points
        {
            // do a cycle:
            ulong ks = SRC(k);
            ulong kd = k;
            tba->set(kd);
            Type t = f[kd];
            while ( ks != k )
            {
                f[kd] = f[ks];
                kd = ks;
                tba->set(kd);
                ks = SRC(ks);
            }
            f[kd] = t;
        }

        if ( 0==ba )  delete tba;
    }
}
// -------------------------
#undef SRC
#undef DST


#endif // !defined HAVE_TRANSPOSE_BA_H__
