#if !defined  HAVE_BITCYCLIC3_H__
#define       HAVE_BITCYCLIC3_H__


#include "fxttypes.h"
#include "bits/bitsperlong.h"
#include "bits/bitrotate.h"
#include "bits/bitcount.h"
#include "bits/bit2pow.h"


inline ulong bit_cyclic_dist(ulong a, ulong b)
// Return minimal bitcount of (t ^ b)
// where t runs through the cyclic rotations.
{
    ulong d = ~0UL;
    ulong t = a;
    do
    {
        ulong z = t ^ b;
        ulong e = bit_count( z );
        if ( e < d )  d = e;
        t = bit_rotate_right(t, 1);
    }
    while ( t!=a );
    return  d;  // not reached
}
// -------------------------

inline ulong bit_cyclic_dist(ulong a, ulong b, ulong ldn)
// Return minimal bitcount of (t ^ b)
// where t runs through the cyclic rotations.
//  using ldn-bit words
{
    ulong d = ~0UL;
    const ulong m = (~0UL>>(BITS_PER_LONG-ldn));
    b &= m;
    a &= m;
    ulong t = a;
    do
    {
        ulong z = t ^ b;
        ulong e = bit_count( z );
        if ( e < d )  d = e;
        t = bit_rotate_right(t, 1, ldn);
    }
    while ( t!=a );
    return  d;  // not reached
}
// -------------------------


inline ulong bit_cyclic_dist1_match(ulong x, ulong y)
// Return  the minimal r so that
//   c = (x ^ rotate_right(y, r, ldn)) is a on-bit word.
// Return ~0UL if there is no such r.
{
    ulong r = 0;
    do
    {
        ulong c = x^y;
        if ( one_bit_q(c) )  return r;
        y = bit_rotate_right(y, 1);
    }
    while ( ++r < BITS_PER_LONG );

    return ~0UL;
}
// -------------------------


inline ulong bit_cyclic_dist1_match(ulong x, ulong y, ulong ldn)
// Return  the minimal r so that
//   c = (x ^ rotate_right(y, r, ldn)) is a on-bit word.
// Return ~0UL if there is no such r.
//  (using ldn-bit words)
{
    ulong r = 0;
    do
    {
        ulong c = x^y;
        if ( one_bit_q(c) )  return r;
        y = bit_rotate_right(y, 1, ldn);
    }
    while ( ++r < ldn );

    return ~0UL;
}
// -------------------------


#endif  // !defined HAVE_BITCYCLIC3_H__
