#if !defined  HAVE_BITNECKLACE_H__
#define       HAVE_BITNECKLACE_H__


#include "fxttypes.h"
#include "bits/bitsperlong.h"
#include "bits/bitcopy.h"
#include "bits/tinyfactors.h"


class bit_necklace
//  binary necklace as binary word
// cf. class binary_prime_string
// The cyclic maximal words are produced.
// Must have 0<n<32.
{
public:
    ulong a_;  // necklace
    ulong m_;  // mask representing n (m==2**(n-1))
    ulong j_;  // mask representing j
    ulong tfb_; // for fast factor lookup

public:
    bit_necklace(ulong n)
    {
        if ( (0==n) || (n>=BITS_PER_LONG) )  n = 1;  // avoid hang
        init(n);
    }

    ~bit_necklace()  { ; }

    void init(ulong n)
    {
        a_ = 0;
        mask_init( 1UL<<(n-1) );
        tfb_ = tiny_factors_tab[n] >> 1;
    }
    void mask_init(ulong m)  { m_ = m; a_ = 0; j_ = 1; }

    ulong data() const { return  a_; }
    ulong current() const { return j_; }

    ulong next()  // return the _mask_ j (zero when finished)
    {
    redo:
        j_ = m_;
        if ( a_ & j_ )  do  { j_>>=1; }  while ( j_ & a_ );

        if ( 0==j_ )  { mask_init(m_);  return 0; }
        a_ |= j_;

        // k initialized to zero for n==BITS_PER_LONG (==> hang)
        for (ulong k=j_<<1,r=1;  k<=m_;  k<<=1,r<<=1)  a_=mask_copy_bit(a_, r, k);

        if ( 0==(tfb_ & j_) )  goto redo;  // necklaces only

        return  j_;
    }

    ulong is_lyndon_word()  const  { return (j_ & m_); }

};
// -------------------------


#endif  // !defined HAVE_BITNECKLACE_H__
