#if !defined  HAVE_FHT_H__
#define       HAVE_FHT_H__


//<<
// General format of arguments: (double *f, ulong ldn)
// f := pointer to data array,
// ldn := base 2 log of array length (length n=2**n)
//
// A function some_func0(...) (note the '0')
// expects input data with higher half zeros ('zero padded' data).
//
// Data in the arrays without const modifier is generally modified.
//
// A comment 'aux' at the line end marks functions that are
// called by other routines and might not be of direct use
// for users of the library
//>>

// fht/fhtdit.cc:
void fht_dit_core(double *f, ulong ldn); // aux
void fht_dit(double *f, ulong ldn);

// fht/fhtdif.cc:
void fht_dif_core(double *f, ulong ldn); // aux
void fht_dif(double *f, ulong ldn);

// fht/cfhtdit.cc:
void fht_dit_core(Complex *f, ulong ldn); // aux
void fht_dit(Complex *f, ulong ldn);

// fht/cfhtdif.cc:
void fht_dif_core(Complex *f, ulong ldn); // aux
void fht_dif(Complex *f, ulong ldn);

// fht/fht0.cc:
void fht0(double *f, ulong ldn);

// fht/cfht0.cc:
void fht0(Complex *f, ulong ldn);

// fht/hartleyshift.cc:
void hartley_shift_05(double *a, ulong n); // aux

// fht/fhtdit2.cc:
void fht_localized_dit2(double *f, ulong ldn);
void fht_dit2(double *f, ulong ldn);

// fht/fhtdif2.cc:
void fht_localized_dif2(double *f, ulong ldn);
void fht_dif2(double *f, ulong ldn);


//: -------- 2-dimensional transform:
// fht/skipfht.cc:
void skip_fht(double *f, ulong n, ulong d, double *w);
void skip_fht0(double *f, ulong n, ulong d, double *w);

// fht/twodimfht.cc:
void row_column_fht(double *f, ulong nr, ulong nc); // aux
void y_transform(double *f, ulong nr, ulong nc); // aux
void twodim_fht(double *f, ulong nr, ulong nc);


//: -------- Spectrum:
// fht/fhtspect.cc:
void fht_spectrum(double *f, ulong ldn, int phasesq=0);


//: -------- Slow algorithms:
// fht/slowht.cc:
void slow_ht(double *f, ulong n);
void slow_ht(Complex *f, ulong n);
void slow_row_column_ht(double *f, ulong nr, ulong nc);
void slow_twodim_ht(double *f, ulong nr, ulong nc);

// fht/recfht2.cc:
void recursive_fht_dit2(double *a, ulong ldn);
void recursive_fht_dif2(double *a, ulong ldn);


#endif  // !defined HAVE_FHT_H__
